/***************************************************************************
                          frontend.cpp  -  description
                             -------------------
    begin                : Die Mr 12 16:36:17 EST 2002
    copyright            : (C) 2002 by Andreas Richter
    email                : ar5@rcs.urz.tu-dresden.de
 ***************************************************************************/

#include <qtextstream.h>
#include <qmultilineedit.h>
#include <qcheckbox.h>
#include <qfileinfo.h>
#include <qbuttongroup.h>
#include <qtable.h>
#include <qlayout.h>

#include <klineedit.h>
#include <kcombobox.h>
#include <kfiledialog.h>
#include <kmessagebox.h>

#include "frontend.h"
#include "options.h"
#include "rcentrystring.h"
#include "myparameterentrydialog.h"

GlobalData *global;
rcEntryString *kk;
KMessageBox *sorryBox;
myParameterEntryDialog *paramEntry;

bool startsolver;
int n_top=5000000;
int n_bottom=1;
int cupmixiv_top=5000000;
int cupmixiv_bottom=1;

Frontend::Frontend( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 ):
TestForm( parent, name, modal, fl )
{
  int i,j;
  global = new GlobalData();
  global->read();
  //locale = new KLocale();
  setFinishEnabled(page(pageCount()-1),true);
  //setNextEnabled(page(1),false);
  for (i=0;i<pageCount();i++) setHelpEnabled(page(i),false);
  w_n->setValidator(new KIntValidator(n_bottom,n_top,this,10,"n"));
  w_cupmixiv->setValidator(new KIntValidator(cupmixiv_bottom,cupmixiv_top,this,10,"n"));
  // create missing Widgets (the ones that didn't make it into QT-Designer)
  DataTable2 = new DataSetTable( Frame4, "DataTable" );
  DataTable2->setGeometry( QRect( 5, 5, 650, 360 ) );
  DataTable2->setFrameShape( QTable::StyledPanel );
  DataTable2->setResizePolicy( QTable::Default );
  DataTable2->setNumRows( global->runsNumber );
  DataTable2->setNumCols( global->variablesNr );
  DataTable2->setRowMovingEnabled( TRUE );
  DataTable2->setColumnMovingEnabled( FALSE );
  Frame4Layout->addWidget( DataTable2, 0, 0 );

  DataTable2->horizontalHeader()->setLabel(0,"n",-1);
  DataTable2->horizontalHeader()->setLabel(1,"D t",-1);
  DataTable2->horizontalHeader()->setLabel(2,"cupmixiv",-1);
  DataTable2->horizontalHeader()->setLabel(3,"A0a",-1);
  DataTable2->horizontalHeader()->setLabel(4,"A0r",-1);
  DataTable2->horizontalHeader()->setLabel(5,"A11",-1);
  DataTable2->horizontalHeader()->setLabel(6,"A12",-1);
  DataTable2->horizontalHeader()->setLabel(7,"A2",-1);
  DataTable2->horizontalHeader()->setLabel(8,"A0Ae",-1);
  DataTable2->horizontalHeader()->setLabel(9,"A0Re",-1);
  DataTable2->horizontalHeader()->setLabel(10,"Re",-1);
  DataTable2->horizontalHeader()->setLabel(11,"r0 [m]",-1);
  DataTable2->horizontalHeader()->setLabel(12,"C01",-1);
  DataTable2->horizontalHeader()->setLabel(13,"C02",-1);
  DataTable2->horizontalHeader()->setLabel(14,"X1",-1);
  DataTable2->horizontalHeader()->setLabel(15,"U1",-1);
  DataTable2->horizontalHeader()->setLabel(16,"X2",-1);
  DataTable2->horizontalHeader()->setLabel(17,"U2",-1);
  DataTable2->horizontalHeader()->setLabel(18,"X3",-1);
  DataTable2->horizontalHeader()->setLabel(19,"U3",-1);
  DataTable2->horizontalHeader()->setLabel(20,"X4",-1);
  DataTable2->horizontalHeader()->setLabel(21,"U4",-1);
  DataTable2->horizontalHeader()->setLabel(22,"X5",-1);
  DataTable2->horizontalHeader()->setLabel(23,"U5",-1);
  DataTable2->horizontalHeader()->setLabel(24,"X6",-1);
  DataTable2->horizontalHeader()->setLabel(25,"U6",-1);
  for(i=0;i<global->variablesNr;i++) {
    DataTable2->setColumnWidth(i,50);
    if (i<10) locations->setColumnWidth(i,50);
    for (j=0;j<DataTable2->numRows();j++) {
    }
  }
  locations->verticalHeader()->setLabel(0,"location",-1);
  locations->verticalHeader()->setLabel(1,"Label",-1);
  locations->verticalHeader()->resizeSection(-1,70);
  connect( DataTable2, SIGNAL( clicked(int,int,int,const QPoint &) ), DataTable2, SLOT( test() ) );
  updateInputFields();
  slotSelectModelType(0);
  pa = new modelParameters();
  outputCompare= new modelOutputComparison();
}

Frontend::~Frontend()
{
  // slotWriteSolverrcFile();
}

void Frontend::slotCreateNewRow()
{
  int i,j;
  DataTable2->setNumRows( DataTable2->numRows()+1 );
  for(i=DataTable2->numRows()-1;i>DataTable2->currentRow();i--)
    for(j=0;j<DataTable2->numCols();j++)
      DataTable2->setText(i,j,DataTable2->text(i-1,j));
  for(j=0;j<DataTable2->numCols();j++)
    DataTable2->setText(DataTable2->currentRow(),j,"");
  if (DataTable2->numRows()==100) InsertNewRowBtn->setEnabled(false);
  DeleteRowBtn->setEnabled(true);
  slotCheckMultipleRunParameters();
}

void Frontend::slotDeleteCurrentRow()
{
  int i,j;
  for(i=DataTable2->currentRow();i<DataTable2->numRows()-1;i++)
    for(j=0;j<DataTable2->numCols();j++)
      DataTable2->setText(i,j,DataTable2->text(i+1,j));
  DataTable2->setNumRows( DataTable2->numRows()-1);
  if (DataTable2->numRows()==1) DeleteRowBtn->setEnabled(false);
  InsertNewRowBtn->setEnabled(true);
  slotCheckMultipleRunParameters();
}

void Frontend::slotMoveTableRow()
{
  int ypos;
  int i,j;
  ypos=DataTable2->currentColumn();
  i = DataTable2->currentRow();
  if (i<DataTable2->numRows()-1) {
    DataTable2->swapRows(i,i+1);
    for(j=0;j<DataTable2->numCols();j++) {
      DataTable2->updateCell(i,j);
      DataTable2->updateCell(i+1,j);
    }
    DataTable2->setCurrentCell(i+1,ypos);
  }
}


void Frontend::slotEditValues()
{
  double params[14];
  int i;
  for(i=0;i<DataTable2->numCols();i++) {
    params[i]=DataTable2->text(DataTable2->currentRow(),i).toDouble();
  }
  paramEntry = new myParameterEntryDialog(this,"paramEntry",true);
  //paramEntry->exec();
  //sorryBox->sorry(this,QString::number(3),"Error!",true);	
  paramEntry->getParameters(params,14,w_flow->currentItem());
  for(i=0;i<DataTable2->numCols();i++) {
    DataTable2->setText(DataTable2->currentRow(),i,QString::number(params[i]));

  }
  /*global->multipleRunVariables[1][3]=params[3];
  DataTable2->setText(DataTable2->currentRow(),3,QString::number(global->multipleRunVariables[1][3]));*/
}


void Frontend::getParameters()
{
  int i,j;
  global->a0a = w_a0a->text().toFloat();
  //global->a0a = locale->readNumber(w_a0a->text(),&okk);
  global->a0r = w_a0r->text().toFloat();
  global->a11 = w_a11->text().toFloat();
  global->a12 = w_a12->text().toFloat();
  global->a2 = w_a2->text().toFloat();
  global->u = w_u->text().toFloat();
  global->l = w_l->text().toFloat();
  global->r0 = w_r0->text().toFloat();
  global->c0 = w_c0->text().toFloat();
  global->d = w_d->text().toFloat();
  global->k1 = w_k1->text().toFloat();
  global->k2 = w_k2->text().toFloat();
  global->vd = w_vd->text().toFloat();
  global->n = w_n->text().toULong();
  global->cupmixiv = w_cupmixiv->text().toFloat();
  global->deltat = w_deltat->text().toFloat();
  global->deltatstar = w_deltat_star->text().toFloat();
  global->re=w_re->text().toFloat();
  global->a0ae=w_a0ae->text().toFloat();
  global->a0re=w_a0re->text().toFloat();
  global->c01 = w_c01->text().toFloat();
	global->c02 = w_c02->text().toFloat();
  // Paths
  global->solverPath = KURL(w_solverPath->url(),0);
  global->initialConditionFileName = KURL(w_initialConditionFileName->url(),0);
  global->outputFileName = KURL(w_outputFileName->url(),0);
  global->cupmixingAverageFileName =KURL(w_cupmixingAverageFileName->url(),0);
  global->plotFileName = KURL(w_plotFileName->url(),0);
  global->plotOptions =w_plotOptions->text();
  global->measurementDataFileName = KURL(w_measurementFileName->url(),0);
  global->measurementPlotFileName = KURL(w_measurementPlotFileName->url(),0);
  global->timeSeriesPlotFileName = KURL(w_timeSeriesPlotFileName->url(),0);
  global->averagetype = w_average_type->currentItem();
  global->flow = w_flow->currentItem();
  global->velocity = w_velocity->isChecked();
  slotCheckMultipleRunParameters();
  for(i=0;i<global->runsNumber;i++)
    for(j=0;j<global->variablesNr;j++) {
      global->multipleRunVariables[i][j] = DataTable2->text(i,j).toFloat();
    }
  slotCheckLocations();
  for(i=0;i<global->locationsNr;i++) {
    global->locations[i] = locations->text(0,i).toFloat();
    global->locationLabels[i] = locations->text(1,i);
  }
}

bool Frontend::checkValidParameters(QString field,double bottom,double top)
{
  bool ok;
  double value;
  value=field.toDouble(&ok);
  if (ok) {
    if (value<bottom) ok=false;
    if (value>top) ok=false;
  }
  return ok;
}

bool Frontend::checkValidParameters(QString field,int bottom,int top)
{
  bool ok;
  int value;
  value=field.toInt(&ok,10);
  if (ok) {
    if (value<bottom) ok=false;
    if (value>top) ok=false;
  }
  return ok;
}

bool Frontend::checkValidPaths(QString path)
{
}

void Frontend::slotWriteSolverrcFile()
{
   getParameters();
   global->write();
   emit(optionsChanged());
}

void Frontend::accept()
{
  slotWriteSolverrcFile();
  done(0);
}


void Frontend::updateInputFields()
{
  int i,j;
  w_a0a->setText(QString::number(global->a0a));
  //w_a0a->setText(locale->formatNumber(QString::number(global->a0a)));
  w_a0r->setText(QString::number(global->a0r));
  w_a11->setText(QString::number(global->a11));
  w_a12->setText(QString::number(global->a12));
  w_a2->setText(QString::number(global->a2));
  w_u->setText(QString::number(global->u));
  w_l->setText(QString::number(global->l));
  w_r0->setText(QString::number(global->r0));
  w_c0->setText(QString::number(global->c0));
  w_d->setText(QString::number(global->d));
  w_k1->setText(QString::number(global->k1));
  w_k2->setText(QString::number(global->k2));
  w_vd->setText(QString::number(global->vd));
  w_a0ae->setText(QString::number(global->a0ae));
  w_a0re->setText(QString::number(global->a0re));
  w_re->setText(QString::number(global->re));
  w_c01->setText(QString::number(global->c01));
  w_c02->setText(QString::number(global->c02));
  w_n->setText(QString::number(global->n));
  w_cupmixiv->setText(QString::number(global->cupmixiv));
  w_deltat->setText(QString::number(global->deltat));
  w_deltat_star->setText(QString::number(global->deltatstar));
  // Paths
  w_solverPath->setURL(global->solverPath.prettyURL());
  w_initialConditionFileName->setURL(global->initialConditionFileName.prettyURL());
  w_outputFileName->setURL(global->outputFileName.prettyURL());
  w_cupmixingAverageFileName->setURL(global->cupmixingAverageFileName.prettyURL());
  w_plotFileName->setURL(global->plotFileName.prettyURL());
  w_plotOptions->setText(global->plotOptions);
  w_measurementFileName->setURL(global->measurementDataFileName.prettyURL());
  w_measurementPlotFileName->setURL(global->measurementPlotFileName.prettyURL());
  w_timeSeriesPlotFileName->setURL(global->timeSeriesPlotFileName.prettyURL());
  // misc
  w_average_type->setCurrentItem(global->averagetype);
  w_flow->setCurrentItem(global->flow);
  slotFlowTypeChanged(global->flow);
  if (w_velocity->isChecked()!=global->velocity) w_velocity->toggle();
  // multiple runs data
  DataTable2->setNumRows(global->runsNumber);
  for(i=0;i<DataTable2->numRows();i++)
    for(j=0;j<global->variablesNr;j++) {
      DataTable2->setText(i,j,QString::number(global->multipleRunVariables[i][j]));
    }
  for(i=0;i<locations->numCols();i++) {
    locations->setText(0,i,QString::number(global->locations[i]));
    locations->setText(1,i,global->locationLabels[i]);
  }

}

void Frontend::next()
{
  QWidget *p;
  int i,j,ppp;
  p = currentPage();
  for(i=0;i<pageCount();i++){
    if (page(i)==p) {
			ppp=i;
			i=pageCount()-1;
		}		
	}
	switch (ppp)	{
	  case 0:
	  {
	    TestForm::next();
	    if (modeltype->id(modeltype->selected())==1) DataTable2->setFocus();
		  break;
	  }
	  case 1:
    {
      global->solverPath = KURL(w_solverPath->url(),0);
      global->initialConditionFileName = KURL(w_initialConditionFileName->url(),0);
      global->outputFileName = KURL(w_outputFileName->url(),0);
      global->cupmixingAverageFileName =KURL(w_cupmixingAverageFileName->url(),0);
      global->plotFileName = KURL(w_plotFileName->url(),0);
      global->measurementDataFileName = KURL(w_measurementFileName->url(),0);
      global->measurementPlotFileName = KURL(w_measurementPlotFileName->url(),0);
      global->averagetype = w_average_type->currentItem();
      global->flow = w_flow->currentItem();
      global->velocity = w_velocity->isChecked();
      TestForm::next();
      break;
    }
    case 2:
    {
  	  global->a0a = w_a0a->text().toFloat();
    	global->a0r = w_a0r->text().toFloat();
   		global->a11 = w_a11->text().toFloat();
	  	global->a12 = w_a12->text().toFloat();
  		global->a2 = w_a2->text().toFloat();
    	global->u = w_u->text().toFloat();
	  	global->l = w_l->text().toFloat();
  		global->r0 = w_r0->text().toFloat();
	  	global->c0 = w_c0->text().toFloat();
  		global->d = w_d->text().toFloat();
    	global->k1 = w_k1->text().toFloat();
	  	global->k2 = w_k2->text().toFloat();
  		global->vd = w_vd->text().toFloat();
  		if (global->flow==1) {
  		  global->re=w_re->text().toFloat();
  		  global->a0ae=w_a0ae->text().toFloat();
  		  global->a0re=w_a0re->text().toFloat();
  		   //sorryBox->sorry(this,QString::number(global->a0ae),"Error!",true);	
  		}
  		global->c01 = w_c01->text().toFloat();
  		global->c02 = w_c02->text().toFloat();
  		global->n = w_n->text().toULong();
      global->cupmixiv = w_cupmixiv->text().toFloat();
      global->deltat = w_deltat->text().toDouble();
    	//  sorryBox->sorry(this,QString::number(global->a0a),"Error!",true);	
    	TestForm::next();
    	break;
	  }
	  case 4:
	  {
	    slotCheckMultipleRunParameters();
      for(i=0;i<global->runsNumber;i++)
        for(j=0;j<global->variablesNr;j++) {
          global->multipleRunVariables[i][j] = DataTable2->text(i,j).toFloat();
        }
      TestForm::next();
      break;	
	  }
	  case 5:
	  {
	    global->timeSeriesPlotFileName = KURL(w_timeSeriesPlotFileName->url(),0);
	    slotCheckLocations();
	    for(i=0;i<global->locationsNr;i++) {
	      global->locations[i]=locations->text(0,i).toFloat();
	      global->locationLabels[i]=locations->text(1,i);
	    }
	    //sorryBox->sorry(this,"help","Error!",true);
	    TestForm::next();
	    break;	
	  }
	  default: TestForm::next();
	}
	currentPage()->setSizePolicy(QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
}

void Frontend::slotCheckMultipleRunParameters()
{
  int i,j;
  for(i=0;i<DataTable2->numRows();i++) {
    for(j=0;j<DataTable2->numCols();j++) {
      if (!DataTable2->text(i,j).isEmpty()) global->runsNumber=i+1;
    }
  }
}

void Frontend::slotCheckLocations()
{
  int i;
  for(i=0;i<locations->numCols();i++)
    if ((locations->text(0,i).isEmpty())||(locations->text(0,i).toDouble()==0)) {
      global->locationsNr=i;
      break;
    }
}


void Frontend::slotSelectModelType(int b)
{
  int i;
  bool sect1,sect2;
  if(b==1) {
    sect2=true;
  }
  else {
    sect2=false;
  }
  sect1=!sect2;
  for(i=2;i<=3;i++) setAppropriate(page(i),sect1);
  for(i=4;i<=6;i++) setAppropriate(page(i),sect2);
  //sorryBox->sorry(this,QString::number(pageCount()-1),"Error!",true);
}

void Frontend::slotValidateDimensionlessParameters()
{
  bool turbulent;
  if(global->flow==1) {
    turbulent=checkValidParameters(w_a0ae->text(),0.0,9e50) &&
    					checkValidParameters(w_a0re->text(),0.0,9e50) &&
    					checkValidParameters(w_re->text(),0.0,9e50) &&
    					checkValidParameters(w_r0->text(),1e-95,9e50);
  }
  else turbulent=true;
  setNextEnabled(page(2),
     (checkValidParameters(w_a0a->text(),0.0,9e50) &&
  	  checkValidParameters(w_a0r->text(),0.0,9e50) &&
  	  checkValidParameters(w_a11->text(),-9e50,9e50) &&
  	  checkValidParameters(w_a12->text(),0.0,9e50) &&
  	  checkValidParameters(w_a2->text(),0.0,9e50) &&
  	  checkValidParameters(w_deltat->text(),1e-99,9e50) && turbulent));
}

void Frontend::slotValidateBasicParameters()
{
  QFileInfo solver(w_solverPath->url());
  QFileInfo iniFile(w_initialConditionFileName->url());
  setNextEnabled(page(1),
  	 ( /*checkValidParameters(w_n->text(),n_bottom,n_top) &&
  	  checkValidParameters(w_cupmixiv->text(),cupmixiv_bottom,cupmixiv_top) && */
  	  solver.isFile() && solver.isExecutable() &&
  	  iniFile.isFile() && iniFile.isReadable()
		 )  	
  	);
}

void Frontend::slotValidateCupmixPlotPossible()
{
  QFileInfo cupmixfile(w_cupmixingAverageFileName->url());
  QFileInfo cupmixdir(KURL(w_plotFileName->url()).directory());
  createCupmixGraphBtn->setEnabled(
     ((cupmixfile.isFile()&&cupmixfile.isWritable())||(cupmixdir.exists()&&!cupmixfile.exists()&&cupmixdir.isWritable()))
  );
}

void Frontend::slotValidateContourPlotPossible()
{
  QFileInfo cupmixfile(w_plotFileName->url());
  QFileInfo cupmixdir(KURL(w_cupmixingAverageFileName->url()).directory());
  createGraphBtn->setEnabled(
     ((cupmixfile.isFile()&&cupmixfile.isWritable())||(cupmixdir.exists()&&!cupmixfile.exists()&&cupmixdir.isWritable()))
  );
}

void Frontend::slotValidateMeasurementPlotPossible()
{
  QFileInfo measurementplotfile(w_measurementPlotFileName->url());
  //QFileInfo cupmixdir(KURL(w_measurePlotFileName->url()).directory());
  createMeasurementPlotBtn->setEnabled(
     ((measurementplotfile.isFile()&&measurementplotfile.isReadable()))
     //||(cupmixdir.exists()&&!cupmixfile.exists()&&cupmixdir.isWritable()))
  );
}

void Frontend::slotCalculateDimensionalParameters()
{
  bool *ok;
	ok = new bool;	
	
  pa->a0a = w_a0a->text().toDouble(ok);
  pa->a0r = w_a0r->text().toDouble(ok);
  pa->a11 = w_a11->text().toDouble(ok);
  pa->a12 = w_a12->text().toDouble(ok);
  pa->a2 = w_a2->text().toDouble(ok);
  pa->dt = w_deltat->text().toDouble(ok);

  pa->r0 = w_r0->text().toDouble(ok);
  pa->c0 = w_c0->text().toDouble(ok);
  pa->l = w_l->text().toDouble(ok);
  pa->u0 = w_u->text().toDouble(ok);
  pa->d = w_d->text().toDouble(ok);
  pa->k1 = w_k1->text().toDouble(ok);
  pa->k2 = w_k2->text().toDouble(ok);
  pa->vd = w_vd->text().toDouble(ok);
  pa->dtstar = w_deltat_star->text().toDouble(ok);

  if (!pa->calculateDimensionalParameters()) {
    sorryBox->sorry(this,"Not enough information available to solve for all missing variables.\nPlease specify some more dimensional data.","Error!",true);
  } else {
    w_r0->setText(QString::number(pa->r0));
    w_c0->setText(QString::number(pa->c0));
    w_l->setText(QString::number(pa->l));
    w_u->setText(QString::number(pa->u0));
    w_d->setText(QString::number(pa->d));
    w_k1->setText(QString::number(pa->k1));
    w_k2->setText(QString::number(pa->k2));
    w_vd->setText(QString::number(pa->vd));
    w_deltat_star->setText(QString::number(pa->dtstar));
  }
}


void Frontend::slotCalculateDimensionlessParameters()
{
  bool *ok;
	ok = new bool;
  pa->d = w_d->text().toDouble(ok);
  pa->k1 = w_k1->text().toDouble(ok);
  pa->k2 = w_k2->text().toDouble(ok);
  pa->vd = w_vd->text().toDouble(ok);
  pa->u0 = w_u->text().toDouble(ok);
  pa->l = w_l->text().toDouble(ok);
  pa->r0 = w_r0->text().toDouble(ok);
  pa->c0 = w_c0->text().toDouble(ok);
  pa->dtstar = w_deltat_star->text().toDouble(ok);
  if (!pa->calculateDimensionlessParameters())
    sorryBox->sorry(this,"U, L,r0 and D have to be known and nonzero!","Error!",true);
  else
  {
    global->a0a=pa->a0a;
    global->a0r=pa->a0r;
    global->a11=pa->a11;
    global->a12=pa->a12;
    global->a2=pa->a2;
    global->deltat=pa->dt;
  }
}

void Frontend::slotFlowTypeChanged(int index)
{
  bool enabled;
  if (index==1) enabled=true; else enabled=false;
  w_a0ae->setEnabled(enabled);
  w_a0re->setEnabled(enabled);
  w_re->setEnabled(enabled);	
}

void Frontend::slotContourPlotProcessExit(KProcess *p)
{
  outputView->insertLine("Contour Plot DONE.");
  p->closeStdout();
  p->~KProcess();
  outputView->setCursorPosition(outputView->numLines(),1);
  createGraphBtn->setEnabled(true);
  createGraphBtn2->setEnabled(true);
}


void Frontend::slotAveragePlotProcessExit(KProcess *p)
{
  outputView->insertLine("Average Plot DONE.");
  p->closeStdout();
  p->~KProcess();
  outputView->setCursorPosition(outputView->numLines(),1);
  createCupmixGraphBtn->setEnabled(true);
  createCupmixGraphBtn2->setEnabled(true);
}

void Frontend::slotMeasurementPlotProcessExit(KProcess *p)
{
  outputView->insertLine("Measurement Plot DONE.");
  p->closeStdout();
  p->~KProcess();
  outputView->setCursorPosition(outputView->numLines(),1);
  createMeasurementPlotBtn->setEnabled(true);
  createMeasurementPlotBtn2->setEnabled(true);
}


void Frontend::slotExtractPlotParameters(KProcess *p)
{
  int len,newpos;
  uint pos;
  // get command line parameters
	len=0;
	pos=0;
	newpos=0;
	while (pos<w_plotOptions->text().length()) {
    newpos=w_plotOptions->text().find(" ",pos,false);	
	  if (newpos<0) len=w_plotOptions->text().length()-pos+1;
		  else len = newpos-pos;
	  *p << w_plotOptions->text().mid(pos,len);		
		pos+=(len+1);
  }
}

void Frontend::slotCreateContourPlot()
{
  QString filename;
  QString errorText;
  contourPlotProcess = new KProcess();
  connect ( contourPlotProcess, SIGNAL(processExited(KProcess *)),this,
                       SLOT(slotContourPlotProcessExit(KProcess *)) );
	contourPlotProcess->clearArguments();
	filename="/usr/X11R6/bin/plotmtv";
	*contourPlotProcess << filename;	
	slotExtractPlotParameters(contourPlotProcess);		
	*contourPlotProcess<<global->plotFileName.path();
	// execute solver program in a konsole window		
	outputView->insertLine("CREATING CONTOUR PLOT...");
	outputView->insertLine(filename);	
	connect ( contourPlotProcess, SIGNAL(receivedStdout(KProcess *, char *, int)),
                 this, SLOT(slotGetOptionsFromStdout(KProcess *, char *, int)));
  //connect
  if (!contourPlotProcess->start(KProcess::NotifyOnExit, KProcess::Stdout )) {
    errorText="Executing \n";
    errorText.append(filename);
    errorText.append("\n not successful!");
    sorryBox->sorry(this,errorText,"Error!",true);
  }
  else {
    createGraphBtn->setEnabled(false);
    createGraphBtn2->setEnabled(false);
  }
}


void Frontend::slotCreateCupmixPlot()
{
  QString filename;
  QString errorText;
  contourPlotProcess = new KProcess();
  connect ( contourPlotProcess, SIGNAL(processExited(KProcess *)),this,
                       SLOT(slotAveragePlotProcessExit(KProcess *)) );
	contourPlotProcess->clearArguments();
	filename="/usr/X11R6/bin/plotmtv";
	*contourPlotProcess << filename;
	// get command line parameters	
	slotExtractPlotParameters(contourPlotProcess);		
	*contourPlotProcess<<global->cupmixingAverageFileName.path();
	// execute solver program in a konsole window		
	outputView->insertLine("CREATING AVERAGES PLOT...");
	outputView->insertLine(filename);	
	connect ( contourPlotProcess, SIGNAL(receivedStdout(KProcess *, char *, int)),
                 this, SLOT(slotGetOptionsFromStdout(KProcess *, char *, int)));
  //connect
  if (!contourPlotProcess->start(KProcess::NotifyOnExit, KProcess::Stdout )) {
    errorText="Executing \n";
    errorText.append(filename);
    errorText.append("\n not successful!");
    sorryBox->sorry(this,errorText,"Error!",true);
  }
  else {
    createCupmixGraphBtn->setEnabled(false);
    createCupmixGraphBtn2->setEnabled(false);
  }
}

void Frontend::slotCreateMeasurementPlot()
{
  QString filename;
  QString errorText;
  measurementPlotProcess = new KProcess();
  connect ( measurementPlotProcess, SIGNAL(processExited(KProcess *)),this,
                       SLOT(slotMeasurementPlotProcessExit(KProcess *)) );
	measurementPlotProcess->clearArguments();
	filename="/usr/X11R6/bin/plotmtv";
	*measurementPlotProcess << filename;
	// get command line parameters	
	slotExtractPlotParameters(measurementPlotProcess);
	*measurementPlotProcess<<global->measurementPlotFileName.path();
	// execute solver program in a konsole window		
	outputView->insertLine("CREATING MEASUREMENT PLOT...");
	outputView->insertLine(filename);	
	connect ( measurementPlotProcess, SIGNAL(receivedStdout(KProcess *, char *, int)),
                 this, SLOT(slotGetOptionsFromStdout(KProcess *, char *, int)));
  //connect
  if (!measurementPlotProcess->start(KProcess::NotifyOnExit, KProcess::Stdout )) {
    errorText="Executing \n";
    errorText.append(filename);
    errorText.append("\n not successful!");
    sorryBox->sorry(this,errorText,"Error!",true);
  }
  else {
    createMeasurementPlotBtn->setEnabled(false);
    createMeasurementPlotBtn2->setEnabled(false);
  }
}

void Frontend::slotGetOptionsFromStdout(KProcess *, char *buffer, int len)
{
  QString line;
  QString outString = QString ( buffer ).append ( "\n" );
  outputView->insertLine(outString);
}

void Frontend::slotSolverProcessExit(KProcess *p)
{
  QFileInfo measurePlot(w_measurementPlotFileName->url());
  QFileInfo contourPlot(w_plotFileName->url());
  QFileInfo cupmixPlot(w_cupmixingAverageFileName->url());
  outputView->insertLine("DONE.");
  p->closeStdout();
  p->~KProcess();
  outputView->setCursorPosition(outputView->numLines(),1);
  startSingleRunBtn->setEnabled(true);
  createMeasurementPlotBtn->setEnabled(measurePlot.isFile() && measurePlot.isReadable());
  createGraphBtn->setEnabled(contourPlot.isFile() && contourPlot.isReadable());
  createCupmixGraphBtn->setEnabled(cupmixPlot.isFile() && cupmixPlot.isReadable());
}

void Frontend::slotGetLocationsFromFile()
{
  int i;
  measurementFile *mData = new measurementFile();
  mData->setFileName(global->measurementDataFileName);
  mData->load();
  for(i=0;i<mData->locationsNr;i++)
  {
    locations->setText(0,i,QString::number(mData->locations[i]));
    global->locations[i]=mData->locations[i];
  }
}

void Frontend::slotGetOptionsMultipleRunsFromStdout(KProcess *, char *buffer, int len)
{
  int i;
  QString line="";
  for(i=0;i<len;i++) line.append(buffer[i]);
  outputView_multipleruns->insert(line);
//  outputView_multipleruns->cursorLeft();
  programOutput.append(line);
}


void Frontend::extractRunResults(int r)
{
  int j=0;
  int s_start=0;
  int s_end=0;
  int s_pos=0;
  int s_pos_old=0;
  QString indicatorString="Exported average concentrations: ";
  QString values;
  QString extractedValue;
  // Look for chararcteristic string in output
  s_start=programOutput.find(indicatorString,0);
  if (s_start>0) {
    // look for end of output values
    s_end=programOutput.find("\n",s_start);
    if (s_end>0) {
      s_start+=indicatorString.length();
      // extract values
      values=programOutput.mid(s_start,s_end-s_start);
      // convert values & store them
      if (values.length()>0) {
        while(values.find(" ",s_pos_old)>0) {
          s_pos=values.find(" ",s_pos_old);
          extractedValue=values.mid(s_pos_old,s_pos-s_pos_old);
          //sorryBox->sorry(this,extractedValue,QString::number(r),true);
          //conversion
          if (j<10) {
            //Ccup[r][j]=extractedValue.toDouble();
            outputCompare->setData(r,j,extractedValue.toDouble());
            s_pos_old=s_pos+1;
            j++;
          }
        }
      }
    }
  }
}

void Frontend::slotMultipleRunsSolverProcessExit(KProcess *p)
{
  int i;
  p->closeStdout();
  p->~KProcess();
  outputView_multipleruns->insert("RUN ");
  outputView_multipleruns->insert(QString::number(runNr+1));
  outputView_multipleruns->insert(" DONE.\n");
  for(i=0;i<80;i++)
    outputView_multipleruns->insert("-");
  outputView_multipleruns->insert("\n");
  outputView_multipleruns->setCursorPosition(outputView_multipleruns->numLines(),1);
  extractRunResults(runNr);
  runNr++;
  startAnotherRun();
}

void Frontend::slotStartBtnClicked()
{
  QString filename;
  kk = new rcEntryString();
  filename=global->solverPath.directory();
  filename.append("/solver.rc");
  QFile f(filename);
	if ( f.open(IO_WriteOnly) ) {    // file opened successfully
	  QTextStream t( &f );           // use a text stream
    kk->rcEntryName="inputfile";
    kk->rcEntryValue=global->initialConditionFileName.path();
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="outputfile";
    kk->rcEntryValue=global->outputFileName.path();
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="tecplotfile";
    kk->rcEntryValue=global->plotFileName.path();
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="a0a";
    kk->rcEntryValue=QString::number(global->a0a);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="a0r";
    kk->rcEntryValue=QString::number(global->a0r);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="a11n";
    kk->rcEntryValue=QString::number(global->a11);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="a12n";
    kk->rcEntryValue=QString::number(global->a12);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="a2";
    kk->rcEntryValue=QString::number(global->a2);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="cupmixfile";
    kk->rcEntryValue=global->cupmixingAverageFileName.path();
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="iterations";
    kk->rcEntryValue=QString::number(global->n);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="timestep";
    kk->rcEntryValue=QString::number(global->deltat);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="cupmixiv";
    kk->rcEntryValue=QString::number(global->cupmixiv);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="average";
    kk->rcEntryValue=w_average_type->text(global->averagetype);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="flow";
    kk->rcEntryValue=w_flow->text(global->flow);
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="velocity";
    if(w_velocity) kk->rcEntryValue="1";
      else kk->rcEntryValue="0";
    t.operator<<(kk->rcEntry());
    kk->rcEntryName="MEASUREMENTFILE";
    kk->rcEntryValue=global->measurementDataFileName.path();
    t.operator<<(kk->rcEntry());
		kk->rcEntryName="MEASUREMENTPLOTFILE";
    kk->rcEntryValue=global->measurementPlotFileName.path();
    t.operator<<(kk->rcEntry());
    if (global->flow==1) {
		  kk->rcEntryName="re";
      kk->rcEntryValue=QString::number(global->re);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="a0ae";
      kk->rcEntryValue=QString::number(global->a0ae);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="a0re";
      kk->rcEntryValue=QString::number(global->a0re);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="r0";
      kk->rcEntryValue=QString::number(global->r0);
      t.operator<<(kk->rcEntry());
      //sorryBox->sorry(this,QString::number(global->a0ae),"Error!",true);
    }
    if (global->c01>0) {
		  kk->rcEntryName="c01";
      kk->rcEntryValue=QString::number(global->c01);
      t.operator<<(kk->rcEntry());
      if (global->c02>0) {
        kk->rcEntryName="c02";
        kk->rcEntryValue=QString::number(global->c02);
        t.operator<<(kk->rcEntry());
      }
    }
	  f.close();
	}
	else {	
	}
	// execute solver program
	filename=global->solverPath.path();
	//filename.append("solver");
	outputView->insert("EXECUTING....\n");
	outputView->insert(filename);
	outputView->insert("\n");
	solverProcess = new KProcess();
  connect ( solverProcess, SIGNAL(processExited(KProcess *)),this,
                       SLOT(slotSolverProcessExit(KProcess *)) );
	solverProcess->clearArguments();
	*solverProcess <<filename;
	connect ( solverProcess, SIGNAL(receivedStdout(KProcess *, char *, int)),
                 this, SLOT(slotGetOptionsFromStdout(KProcess *, char *, int)));
  if (!solverProcess->start(KProcess::NotifyOnExit, KProcess::Stdout ))
  { }
  else startSingleRunBtn->setEnabled(false);
}

void Frontend::modifyInputFile(QString filename, int runNr)
{
  QString s;
  double xcoord;
  double xold=-0.1;
  double u_new;
  QString xcoord_s,u_s;
  QString u_s_new;
  int xpos_end=0;
  int upos_begin, upos_end;
  int i;
  int start_x=14;
  QString filename_mod;
  QFile f(filename);
  filename_mod=filename;
  filename_mod.append("_temp");
  QFile o(filename_mod);
  if ( f.open(IO_ReadOnly)&& o.open(IO_WriteOnly )) {    // file opened successfully
    QTextStream t( &f );           											// use a text stream
    QTextStream ot( &o);
    for(i=1;i<4;i++) {
      s=t.readLine();
      ot<<s.append("\n");
    }
    while ( !t.eof() ) {        // until end of file...
      s = t.readLine();       // line of text excluding '\n'
      xpos_end=s.find(" ");
      if (xpos_end>0) {
        xcoord_s=s.mid(0,xpos_end);
        upos_begin=s.findRev(" ")+1;
        upos_end=s.length();
        u_s=s.mid(upos_begin,upos_end-upos_begin+1);
        xcoord=xcoord_s.toDouble();
        if ((xold<xcoord)) {
          for(i=start_x;i<global->variablesNr;i+=2) {
            if((xold<global->multipleRunVariables[runNr][i])&&
               (global->multipleRunVariables[runNr][i]<=xcoord))
            {
              u_new = global->multipleRunVariables[runNr][i+1];
              u_s_new=QString::number(u_new);
              start_x+=2;
              //sorryBox->sorry(this,u_s_new,QString::number(i+1),true);
              break;
            }
          }
          xold=xcoord;
        }
        s.replace(upos_begin,upos_end-upos_begin,u_s_new);
        ot<<s.append("\n");
        /*if (runNr==1) {
          sorryBox->sorry(this,u_s_new,QString::number(upos_begin)+" "+QString::number(upos_end-upos_begin),true);
        }*/
      }
    }
    f.close();
    o.close();
    filename_mod=filename;
    filename_mod.append(".bak");
    QDir path;
    path.rename(filename,filename_mod);
    filename_mod=filename;
    filename_mod.append("_temp");
    path.rename(filename_mod,filename);
  }
  else {
    s="Modification of Inputfile\n";
    s.append(filename);
    s.append("\n was not successful!");
    sorryBox->sorry(this,s,"Error",true);
  }
}

void Frontend::startAnotherRun()
{
  double time=0;
  int i;
  bool fileCreationSuccessful=true;
  QString filename, filename2;
  QString outputText;
  programOutput="";
  if (runNr<global->runsNumber) {
    time+=(global->multipleRunVariables[runNr][0]*global->multipleRunVariables[runNr][1]);
    /*if (runNr<1) cupTime[runNr]=time;
      else cupTime[runNr]=time+cupTime[runNr-1];*/
    if (runNr<1) outputCompare->times[runNr]=time;
      else outputCompare->times[runNr]=time+outputCompare->times[runNr-1];
    kk = new rcEntryString();
    filename=global->solverPath.directory();
    filename.append("/solver.rc");
    QFile f(filename);
 	  if ( f.open(IO_WriteOnly) ) {    // file opened successfully
 	    QTextStream t( &f );        // use a text stream
      kk->rcEntryName="inputfile";
      filename = global->initialConditionFileName.path();
      if(runNr>0) {
        filename.append("_");
        filename.append(QString::number(runNr-1));
      }
      kk->rcEntryValue=filename;
      t.operator<<(kk->rcEntry());
      if (global->velocity) {
        outputView_multipleruns->insert("Modifying input file ");
        outputView_multipleruns->insert(filename);
        outputView_multipleruns->insert(".\n");
        modifyInputFile(filename, runNr);
      }
      kk->rcEntryName="outputfile";
      if (runNr<global->runsNumber-1) {
        filename=global->initialConditionFileName.path();
        filename.append("_");
        filename.append(QString::number(runNr));
      }
      else {
        filename=global->outputFileName.path();
      }
      kk->rcEntryValue=filename;
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="tecplotfile";
      kk->rcEntryValue=global->plotFileName.path();
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="a0a";
      kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][3]);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="a0r";
      kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][4]);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="a11n";
      kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][5]);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="a12n";
      kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][6]);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="a2";
      kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][7]);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="cupmixfile";
      filename=global->cupmixingAverageFileName.path();
      filename.append(QString::number(runNr));
      kk->rcEntryValue=filename;
      t.operator<<(kk->rcEntry());
      //
      filename=global->cupmixingAverageFileName.path();
      filename.append(QString::number(runNr));
      /*kk->rcEntryName="MULTIPLERUNSPLOTFILENAME";
      kk->rcEntryValue=filename;
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="PLOTLABEL";
      filename="T=";
      filename.append(QString::number(time));
      kk->rcEntryValue=filename;
      t.operator<<(kk->rcEntry());*/
      kk->rcEntryName="run";
      kk->rcEntryValue=QString::number(runNr+1);
      t.operator<<(kk->rcEntry());
      //
      kk->rcEntryName="iterations";
      kk->rcEntryValue=QString::number((int)(global->multipleRunVariables[runNr][0]));
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="timestep";
      kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][1]);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="cupmixiv";
      kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][2]);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="average";
      kk->rcEntryValue=w_average_type->text(global->averagetype);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="flow";
      kk->rcEntryValue=w_flow->text(global->flow);
      t.operator<<(kk->rcEntry());
      kk->rcEntryName="velocity";
      if(w_velocity) kk->rcEntryValue="1";
        else kk->rcEntryValue="0";
      t.operator<<(kk->rcEntry());
      if (global->flow==1) {
 	 	    kk->rcEntryName="re";
        kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][10]);
        t.operator<<(kk->rcEntry());
        kk->rcEntryName="a0ae";
        kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][8]);
        t.operator<<(kk->rcEntry());
        kk->rcEntryName="a0re";
        kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][9]);
        t.operator<<(kk->rcEntry());
        kk->rcEntryName="r0";
        kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][11]);
        t.operator<<(kk->rcEntry());
      }
      if (global->multipleRunVariables[runNr][12]>0) {
 	      kk->rcEntryName="c01";
        kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][12]);
        t.operator<<(kk->rcEntry());
        if (global->multipleRunVariables[runNr][13]>0) {
          kk->rcEntryName="c02";
          kk->rcEntryValue=QString::number(global->multipleRunVariables[runNr][13]);
          t.operator<<(kk->rcEntry());
        }
      }
      kk->rcEntryName="EXPORTAVERAGEAT";
      for(i=0;i<global->locationsNr;i++) {
        kk->rcEntryValue=QString::number(global->locations[i]);
        t.operator<<(kk->rcEntry());
      }
 	    f.close();
      // execute solver program
   	  filename=global->solverPath.path();
   	  outputText="EXECUTING run no. ";
   	  outputText.append(QString::number(runNr+1));
   	  outputView_multipleruns->insert(outputText);
   	  outputView_multipleruns->insert("\n");
   	  outputView_multipleruns->insert(filename);
   	  outputView_multipleruns->insert("\n");
   	  solverProcess = new KProcess();
      connect ( solverProcess, SIGNAL(processExited(KProcess *)),this,
                         SLOT(slotMultipleRunsSolverProcessExit(KProcess *)) );
   	  solverProcess->clearArguments();
   	  *solverProcess <<filename;
   	  connect ( solverProcess, SIGNAL(receivedStdout(KProcess *, char *, int)),
                     this, SLOT(slotGetOptionsMultipleRunsFromStdout(KProcess *, char *, int)));
      solverProcess->start(KProcess::NotifyOnExit, KProcess::AllOutput );
 	  }  	
  }
  else {
    // do not start another run, leave loop
    startMultipleRunsBtn->setEnabled(true);
    QFileInfo measurePlot(w_measurementPlotFileName->url());
    QFileInfo contourPlot(w_plotFileName->url());
    QFileInfo cupmixPlot(w_cupmixingAverageFileName->url());
    createMeasurementPlotBtn2->setEnabled(measurePlot.isFile() && measurePlot.isReadable());
    createGraphBtn2->setEnabled(contourPlot.isFile() && contourPlot.isReadable());
    createCupmixGraphBtn2->setEnabled(cupmixPlot.isFile() && cupmixPlot.isReadable());
    //concentrationsWithTimePlot();
    fileCreationSuccessful=outputCompare->compare(global->measurementDataFileName);
    if (fileCreationSuccessful) {
      for(i=0;i<outputCompare->locationsNr;i++) {
        filename=global->timeSeriesPlotFileName.path();
        filename.append(".");
        filename.append(outputCompare->locationLabels[i]);
        if (!outputCompare->plotObservedVsPredictedCurve(i,KURL::KURL(filename))) {
          outputText="Plot for location ";
          outputText.append(QString::number(outputCompare->locations[i]));
          outputText.append(" cannot be created.\n");
          outputView_multipleruns->insert(outputText);
          sorryBox->sorry(this,outputText,"Error",true);
        }
        else {
          filename.append(".residuals");
          outputCompare->plotResidualsPlot(i,KURL::KURL(filename));
          filename2=filename;
          outputCompare->plotResidualsProbabilityPlot(i,KURL::KURL(filename2.append("Probability")));
          filename2=filename;
          outputCompare->plotResidualsHistogram(i,KURL::KURL(filename2.append("Histogram")));
        }
        filename=global->timeSeriesPlotFileName.path();
        if (!outputCompare->plotAllCurves(KURL::KURL(filename))) {
          outputText="Plot ";
          outputText.append(filename);
          outputText.append(" cannot be created.\n");
          outputView_multipleruns->insert(outputText);
          sorryBox->sorry(this,outputText,"Error",true);
        }
      }
      //delete outputCompare data;
      outputCompare->clearData();
    }
    else {
      outputText="Comparison cannot be executed\n- time values in measurement file and calculation are different!\n";
      outputView_multipleruns->insert(outputText);
      sorryBox->sorry(this,outputText,"Error",true);
    }
  }
}

void Frontend::slotStartMultipleRunsBtnClicked()
{
  startMultipleRunsBtn->setEnabled(false);
  outputCompare->setLocations(global->locations,global->locationLabels);
  runNr=0;
  startAnotherRun();
}