/***************************************************************************
                          frontend.h  -  description
                             -------------------
    begin                : Die Mr 12 16:36:17 EST 2002
    copyright            : (C) 2002 by Andreas Richter
    email                : ar5@rcs.urz.tu-dresden.de
 ***************************************************************************/

#ifndef FRONTEND_H
#define FRONTEND_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>

#include <kconfig.h>
#include <kprocess.h>
#include <knumvalidator.h>

#include <kurlrequester.h>

#include "input_interface.h"
#include "datasettable.h"
#include "modelparameters.h"

#include "modeloutputcomparison.h"

class QPopupMenu;

class Frontend : public TestForm
{
 Q_OBJECT
 public:
    /** constructor */
    Frontend( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
/*    Frontend( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 ):
    TestForm( parent, name, modal, fl ); {} */
    /** destructor */
    ~Frontend();
		
 signals:
    void optionsChanged();

  public slots:
    virtual void slotWriteSolverrcFile();
    virtual void accept();
    //
    virtual void slotCalculateDimensionlessParameters();
    virtual void slotCalculateDimensionalParameters();
    virtual void next();
    //
    virtual void slotSelectModelType(int);
    //
    virtual void slotStartBtnClicked();
    virtual void slotCreateContourPlot();
    virtual void slotCreateCupmixPlot();
    virtual void slotCreateMeasurementPlot();
    //
    virtual void slotFlowTypeChanged(int);
    //
    virtual void slotCreateNewRow();
    virtual void slotDeleteCurrentRow();
    virtual void slotMoveTableRow();
    virtual void slotEditValues();
    //
    virtual void slotStartMultipleRunsBtnClicked();
    // validation slots
    virtual void slotValidateDimensionlessParameters();
    virtual void slotValidateBasicParameters();
    virtual void slotValidateContourPlotPossible();
    virtual void slotValidateCupmixPlotPossible();
    virtual void slotValidateMeasurementPlotPossible();		

    virtual void slotGetLocationsFromFile();

/*  	void slotCalculateDimensionalParameters();
  	void slotCalculateDimensionlessParameters();
    void slotGetDirectoryName();
    void slotGetInitialFileName();
    void slotGetOutputFileName();
    void slotGetCupmixingAverageFileName();
    void slotGetTecplotFileName(); */

  private slots:
    void slotExtractPlotParameters(KProcess *);
    void slotGetOptionsFromStdout(KProcess *, char *buffer, int len);
    void slotSolverProcessExit(KProcess *);
    void slotGetOptionsMultipleRunsFromStdout(KProcess *, char *buffer, int len);
    void slotMultipleRunsSolverProcessExit(KProcess *);
    void slotContourPlotProcessExit(KProcess *);
    void slotMeasurementPlotProcessExit(KProcess *);
    void slotAveragePlotProcessExit(KProcess *);

    void slotCheckMultipleRunParameters();
    void startAnotherRun();
    void extractRunResults(int);

    void slotCheckLocations();

    void modifyInputFile(QString, int);

  private:
    KProcess *solverProcess;
    KProcess *contourPlotProcess;
    KProcess *cupmixPlotProcess;
    KProcess *measurementPlotProcess;

    DataSetTable *DataTable2;
    modelParameters *pa;
    int runNr;
    QString programOutput;

    // !!!!!!
    double Ccup[20][10];
    double cupTime[20];

    // !!!
    modelOutputComparison *outputCompare;

  protected:
    void getParameters();
    void updateInputFields();
    bool checkValidParameters(QString,double,double);
    bool checkValidParameters(QString,int,int);
    bool checkValidPaths(QString);

};


#endif
