/***************************************************************************
                          modeloutputcomparison.cpp  -  description
                             -------------------
    begin                : Mon Apr 8 2002
    copyright            : (C) 2002 by Andreas Richter
    email                : ar5@rcs.urz.tu-dresden.de
 ***************************************************************************/

#include <qfile.h>
#include <qtextstream.h>

#include "modeloutputcomparison.h"

modelOutputComparison::modelOutputComparison(){
  mData = new measurementFile();
  mDataLoaded=false;
  datasetNr=0;
  locationsNr=0;
}
modelOutputComparison::~modelOutputComparison(){
  mData->~measurementFile();
  // delete mData;
}

bool modelOutputComparison::calculateResiduals()
{
  int i,j;
  for(i=0;i<locationsNr;i++)
    for(j=0;j<datasetNr;j++)
      residuals[j][i]=mData->data[j][matches[i]]-data[j][i];
  return true;
}

double modelOutputComparison::average(double d[100], int N)
{
  int i;
  double sum=0;
  for(i=0;i<N;i++) sum+=d[i];
  return (sum/((double)N));
}

double modelOutputComparison::diffErrorSquares(double d1[100],double d, int N)
{
  double diff;
  double sum=0;
  int i;
  for(i=0;i<N;i++) {
    diff=d1[i]-d;
    sum+=diff*diff;
  }
  return sum;
}

double modelOutputComparison::getCovariance(double d1[100], double d2[100], int N)
{
  int i;
  double sum=0;
  double diff=0;
  for (i=0;i<N;i++) {
    diff=d1[i]-d2[i];
    sum+=diff*diff;
  }
  return (sum/((double)(N-1)));
}

void modelOutputComparison::clearData()
{
  int i,j;
  for(i=0;i<locationsNr;i++) {
    for(j=0;j<datasetNr;j++) {
      data[j][i]=0;
    }
    locations[i]=0;
    covariance[i]=0;
    B[i]=0;
  }
  for(j=0;j<datasetNr;j++) {
    times[j]=0;
  }
  locationsNr=0;
  datasetNr=0;
}

bool modelOutputComparison::compare(KURL mFileName)
{
  int i,j;
  double y_bar;
  double sum1, sum2;
  bool returnvalue=true;
  mData->setFileName(mFileName);
  mDataLoaded=mData->load();
  for(j=0;j<locationsNr;j++) matchfound[j]=0;
  for(i=0;i<mData->locationsNr;i++) {
    if (fabs((mData->locations[i]-locations[i])/mData->locations[i])<0.01) {
        matches[i]=i;
        matchfound[i]=true;
    } else {
      for(j=0;j<locationsNr;j++)
        if (fabs((mData->locations[i]-locations[j])/mData->locations[i])<0.01) {
          matches[j]=i;
          matchfound[j]=true;
        }
    }
  }
  for(j=0;j<datasetNr;j++)
    if (fabs((times[j]-mData->times[j])/times[j])>=0.01)
      returnvalue=false;
  if (returnvalue==true) {
    //calculate residuals
    calculateResiduals();
    calculateClasses();
    for(i=0;i<locationsNr;i++) {
      if (matchfound[i]) {
        // store data in vectors
        for(j=0;j<datasetNr;j++) {
          measure[j]=mData->data[j][matches[i]];
          calculated[j]=data[j][i];
        }
        // calculate average
        y_bar = average(measure,mData->datasetNr);
        //calculate covariance
        covariance[i]=getCovariance(measure,calculated,datasetNr);
        // calculate B
        sum1 = diffErrorSquares(calculated,y_bar,datasetNr);
        sum2 = diffErrorSquares(measure, y_bar,datasetNr);
        B[i]= sum1/sum2;
      }
      else {
        B[i]=-1;
        covariance[i]=-1;
        returnvalue=false;
      }
    }
  }
  return returnvalue;
}

void modelOutputComparison::calculateClasses()
{
  int i,j;
  double minResidual, maxResidual;
  double spacing;
  for(i=0;i<locationsNr;i++) {
    minResidual=0;
    maxResidual=0;
    for(j=0;j<datasetNr;j++) {
      if (residuals[j][i]<minResidual) minResidual=residuals[j][i];
      else
        if (residuals[j][i]>maxResidual) maxResidual=residuals[j][i];
    }
    residualClasses[0][i]=minResidual;
    residualClasses[10][i]=maxResidual;
    spacing=fabs((maxResidual-minResidual)/(10.0));
    for(j=1;j<10;j++) {
      residualClasses[j][i]=minResidual+(spacing*((double)(j)));
    }
  }
}

void modelOutputComparison::calculateResidualDistribution()
{
  int i,j,k;
  double value;
  for(i=0;i<locationsNr;i++) {
    // reset distribution matrix
    for(k=0;k<10;k++)
      residualsDistribution[k][i]=0;
    // for each residual...
    for(j=0;j<datasetNr;j++) {
      value=residuals[j][i];
      // find matching class...
      for(k=1;k<11;k++) {
        if((residualClasses[k-1][i]<=value)&&(value<residualClasses[k][i])) {
          // and increase corresponding class counter
          residualsDistribution[k-1][i]++;
        }
      }
    }
  }
}

void modelOutputComparison::setLocations(double l[10], QString lLabels[10])
{
  int i;
  for(i=0;i<10;i++) {
    locations[i]=l[i];
    locationLabels[i]=lLabels[i];
    if(locations[i]>0) locationsNr=i+1;
  }
}

void modelOutputComparison::setTimes(double t[100])
{
  int i;
  for(i=0;i<100;i++) {
    times[i]=t[i];
  }
}

void modelOutputComparison::setData(int i, int j, double value)
{
  data[i][j]=value;
  if(i+1>datasetNr) datasetNr=i+1;
}

bool modelOutputComparison::plotAllCurves(KURL filename)
{
  int i,j;
  bool returnError=false;
  QFile f(filename.path());
  if ( f.open(IO_WriteOnly) ) {    // file opened successfully
	  QTextStream t( &f );           // use a text stream
	  t<<"$ DATA=CURVE2D\n";
	  t<<"% toplabel = \"Cup-Mixing Average Chlorine Concentration with time for discrete locations along the pipe\"\n";
	  t<<"% xlabel = \"T\"\n";
    t<<"% ylabel = \"C\"\n";
    t<<"% xmin=0\n";
    t<<"% ymin=0 ymax=1.1\n";
    for(j=0;j<locationsNr;j++)
    {
      t<<"% linelabel=\"";
      t<<locationLabels[j];
      t<<"\"\n";
      t<<"% linetype=";
      t<<QString::number(j+1);
      t<<"\n";
      for(i=0;i<datasetNr;i++){
        t<<" "<<QString::number(times[i])<<" "<<QString::number(data[i][j])<<"\n";
      }
      t<<"\n";
    }
	}
	else { returnError=true; }
	f.close();
	
	return !returnError;
}

bool modelOutputComparison::plotObservedVsPredictedCurve(int curveNr, KURL filename)
{
  int i;
  bool returnerror=false;
  if (matchfound[curveNr]) {
    QFile f(filename.path());
    if ( f.open(IO_WriteOnly) ) {    // file opened successfully
	    QTextStream t( &f );           // use a text stream
	    t<<"$ DATA=CURVE2D\n";
	    t<<"% toplabel = \"Cup-mixing average chlorine concentration - observed and calculated values\"\n";
	    t<<"% toplabel = \""<<locationLabels[curveNr]<<"\n";
	    //t<<", B = "<<QString::number(B[curveNr])<<", covariance: ";
	    t<<"% subtitle = \"covariance: ";
	    t<<QString::number(covariance[curveNr])<<"\"\n";
	    t<<"% xlabel = \"T\"\n";
      t<<"% ylabel = \"C\"\n";
      t<<"% equalscale = false\n";
      t<<"%fitpage = false\n";
      t<<"% xyratio = 0.5\n";
      t<<"% xmin=0\n";
      t<<"% ymin=0 ymax=1.1\n";
      t<<"% linelabel=\"model output\"\n";
      t<<"% linetype=1\n";
      for(i=0;i<datasetNr;i++){
        t<<" "<<QString::number(times[i])<<" "<<QString::number(data[i][curveNr])<<"\n";
      }
      t<<"\n";
      t<<"% linelabel=\"measurement data\"\n";
      t<<"% linetype=0\n";
      t<<"% markertype=2\n";
      for(i=0;i<mData->datasetNr;i++) {
        t<<" "<<QString::number(mData->times[i])<<" ";
        t<<QString::number(mData->data[i][matches[curveNr]])<<"\n";
      }
  	}
  	else {  returnerror=true; }
  	f.close();
  }
  else returnerror=true;

  return !returnerror;
}

bool modelOutputComparison::plotResidualsPlot(int curveNr, KURL filename)
{
  int i;
  double maxerror=0;
  bool returnerror=false;
  if (matchfound[curveNr]) {
    QFile f(filename.path());
    if ( f.open(IO_WriteOnly) ) {    // file opened successfully
      for(i=0;i<datasetNr;i++)
        if (fabs(residuals[i][curveNr])>maxerror) maxerror=fabs(residuals[i][curveNr]);
	    QTextStream t( &f );           // use a text stream
	    t<<"$ DATA=CURVE2D\n";
	    t<<"% toplabel = \"Cup-mixing average chlorine concentration - residuals between observed and calculated values\"\n";
	    t<<"% subtitle = \""<<locationLabels[curveNr]<<"\"\n";
	    t<<"% xlabel = \"T\"\n";
      t<<"% ylabel = \"C\"\n";
      t<<"% ymin="<<QString::number(maxerror*(-1.1))<<" ymax="<<QString::number(maxerror*1.1)<<"\n";
      t<<"% xmin=0\n";
      t<<"% linelabel=\"residuals\"\n";
      t<<"% linetype=1\n";
      t<<"% markertype=3\n";
      for(i=0;i<datasetNr;i++){
        t<<" "<<QString::number(times[i])<<" "<<QString::number(residuals[i][curveNr])<<"\n";
      }
      t<<"\n";
  	}
  	else { }
  	f.close();
  }
  else returnerror=true;

  return !returnerror;
}

bool modelOutputComparison::plotResidualsProbabilityPlot(int curveNr, KURL filename)
{
  int i;
  bool returnerror=false;
  if (matchfound[curveNr]) {
    QFile f(filename.path());
    if ( f.open(IO_WriteOnly) ) {    // file opened successfully
	    QTextStream t( &f );           // use a text stream
	    t<<"$ DATA=PROBABILITY\n";
	    t<<"% toplabel = \"Cup-mixing average chlorine concentration - residuals distribution\"\n";
	    t<<"% subtitle = \""<<locationLabels[curveNr]<<"\"\n";
	    t<<"% xlabel = \"residual value\"\n";
      t<<"% ylabel = \"Integral probability\"\n";
      /*t<<"% xmin=";
      t<<QString::number(classes[0][runNr])<<" xmax="
      t<<"\n";
      t<<"% linelabel=\"residuals\"\n";
      t<<"% linetype=1\n";
      t<<"% markertype=3\n";*/
      for(i=0;i<datasetNr;i++){
        t<<" "<<QString::number(residuals[i][curveNr])<<"\n";
      }
      t<<"\n";
  	}
  	else { }
  	f.close();
  }
  else returnerror=true;

  return !returnerror;
}

bool modelOutputComparison::plotResidualsHistogram(int curveNr, KURL filename)
{
  int i;
  bool returnerror=false;
  if (matchfound[curveNr]) {
    QFile f(filename.path());
    if ( f.open(IO_WriteOnly) ) {    // file opened successfully
	    QTextStream t( &f );           // use a text stream
	    t<<"$ DATA=HISTOGRAM\n";
	    t<<"% toplabel = \"Cup-mixing average chlorine concentration - residuals distribution\"\n";
	    t<<"% subtitle = \""<<locationLabels[curveNr]<<"\"\n";
	    t<<"% xlabel = \"residual value\"\n";
      t<<"% ylabel = \"Frequency\"\n";
      t<<"% xmin=";
      t<<QString::number(residualClasses[0][curveNr])<<" xmax=";
      t<<QString::number(residualClasses[10][curveNr]);
      t<<"\n";
      t<<"% linelabel=\"residuals\"\n";
      t<<"% linetype=1\n";
      t<<"% markertype=3\n";
      for(i=0;i<datasetNr;i++){
        t<<" "<<QString::number(residuals[i][curveNr])<<"\n";
      }
      t<<"\n";
  	}
  	else { }
  	f.close();
  }
  else returnerror=true;

  return !returnerror;
}
