/***************************************************************************
                          modelparameters.cpp  -  description
                             -------------------
    begin                : Wed Apr 3 2002
    copyright            : (C) 2002 by Andreas Richter
    email                : ar5@rcs.urz.tu-dresden.de
 ***************************************************************************/


#include "modelparameters.h"

modelParameters::modelParameters(){
}
modelParameters::~modelParameters(){
}


void modelParameters::calcU()
{
  int caseNr=0;
  if (!((u0>0)&&(fabs(u0*l*a0a-d)<=1e-5)))
    if (!((l>0)&&(d>0)&&(a0a>0)))
      if (!((k1>0)&&(l>0)&&(a11>0)))
        if (!((l>0)&&(dtstar>0)&&(dt>0)))
          if (!((d>0)&&(r0>0)&&(a0r>0)&&(l>0)))
            if (!((k2>0)&&(l>0)&&(c0>0)&&(a12>0)))
              error=true;
            else caseNr=6;
          else caseNr=5;
        else caseNr=4;
      else caseNr=3;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: u0=d/l/a0a; calculated=true; break;
    case 3: u0=k1*l/a11; calculated=true; break;
    case 4: u0=dtstar*l/dt; calculated=true; break;
    case 5: u0=d*l/a0r/r0; calculated=true; break;
    case 6: u0=k2*l*c0/a12; calculated=true; break;
  }
}

void modelParameters::calcL()
{
  int caseNr=0;
  if (!((l>0)&&(fabs(u0*l*a0a-d)<=1e-5)))
    if (!((u0>0)&&(d>0)&&(a0a>0)))
      if (!((k1>0)&&(u0>0)&&(a11>0)))
        if (!((u0>0)&&(dtstar>0)&&(dt>0)))
          if (!((d>0)&&(r0>0)&&(a0r>0)&&(u0>0)))
            if (!((k2>0)&&(u0>0)&&(c0>0)&&(a12>0)))
              error=true;
            else caseNr=6;
          else caseNr=5;
        else caseNr=4;
      else caseNr=3;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: l=d/u0/a0a; calculated=true; break;
    case 3: l=a11*u0/k1; calculated=true; break;
    case 4: l=dt/dtstar*u0; calculated=true; break;
    case 5: l=a0r*u0*r0/d; calculated=true; break;
    case 6: l=a2*u0/c0/k2; calculated=true; break;
  }
}

void modelParameters::calcD()
{
  int caseNr=0;
  if (!(((d>0)&&(fabs(u0*l*a0a-d)<=1e-5)) || (((a0a==0)||(u0==0) )&&(d==0))))
    if (!(l>0))
      if (!((r0>0)&&(a2>0)))
        error=true;
      else caseNr=3;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: d=a0a*u0*l; calculated=true; break;
    case 3: d=vd*r0/a2; calculated=true; break;
  }
}

void modelParameters::calcK1()
{
  int caseNr=0;
  if (!(((k1>0)&&(fabs(k1*l-a11*u0)<=1e-5)) || (((a11==0)||(u0==0))&&(k1==0))))
    if (!((l>0)))
      error=true;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: k1=a11*u0/l; calculated=true; break;
  }
}

void modelParameters::calcK2()
{
  int caseNr=0;
  if (!(((k2>0)&&(fabs(k2*l*c0-a12*u0)<=1e-5)) || (((a12==0)||(u0==0))&&(k2==0))))
    if (!((l>0)&&(c0>0)))
      error=true;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: k2 = a12*u0/l/c0; calculated=true; break;
  }
}

void modelParameters::calcVd()
{
  int caseNr=0;
  if (!(((vd>0)&&(fabs(vd*r0-a2*d)<=1e-5)) ||(((a2==0)||(d==0))&&(vd==0))))
    if (!(r0>0))
      error=true;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: vd = a2*d/r0; calculated=true; break;
  }
}

void modelParameters::calcC0()
{
  int caseNr=0;
  if (!((c0>0)||(((a12==0)||(u0==0))&&(c0==0))))
    if (!((l>0)&&(k2>0)))
      error=true;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: c0 = a12*u0/l/k2; calculated = true; break;
  }
}

void modelParameters::calcR0()
{
  int caseNr=0;
  if (!(r0>0))
    if (!(vd>0))
      if (!((a0r>0)&&(u0>0)))
        error=true;
      else caseNr=3;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: r0 = a2*d/vd; calculated = true; break;
    case 3: r0 = d*l/a0r/u0; calculated = true; break;
  }
}

void modelParameters::calcDeltaTstar()
{
  int caseNr=0;
  if (!((dtstar>0)&&((fabs(dtstar*l-dt*u0)<=1e-5))))
    if (!(u0>0))
      error=true;
    else caseNr=2;
  else caseNr=1;
  switch(caseNr)
  {
    case 1: break;
    case 2: dtstar=dt*l/u0; calculated = true; break;
  }
}

bool modelParameters::calculateDimensionlessParameters()
{
  bool error=false;
  if ((u0==0)||(l==0)||(r0==0)||(d==0)) error=true;
  else
  {
    a0a=d/u0/l;
    a0r=l*d/u0/r0/r0;
    a11=k1*l/u0;
    a12=k2*l/u0*c0;
    a2=vd*r0/d;
    dt=dtstar*l/u0;	
  }
  return !error;
}


bool modelParameters::calculateDimensionalParameters()
{
  int step=1;
  calculated=true;
  error=true;
  while ((error&&calculated)&&(step<20))
  {
    error=false;
    calculated=false;
    calcK1();
    calcK2();
    calcVd();
    calcC0();
    calcD();
    calcR0();
    calcU();
    calcL();
    calcDeltaTstar();
    step++;
  }
  return !error;
}