/* -------------------------------------------------------------

   options.cpp (part of The KDE Dictionary Client)

   Copyright (C) 2000-2001 Christian Gebauer <gebauer@bigfoot.com>

   This file is distributed under the Artistic License.
   See LICENSE for details.

   -------------------------------------------------------------

   GlobalData      manages all global data of Kdict
   
   OptionsDialog   the "Preferences" dialog

   DbSetsDialog    dialog for editing the user defined database sets
   
 ------------------------------------------------------------- */

#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcolor.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qwindowdefs.h>
#include <qcombobox.h>
#include <qkeycode.h>
#include <qlistbox.h>
#include <qbuttongroup.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <qlineedit.h>

#include <kcolordialog.h>
#include <kwin.h>
#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <knumvalidator.h>
#include <knuminput.h>
#include <klineedit.h>
#include <kapp.h>
#include <kcharsets.h>
#include <kfontdialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>

#include "options.h"

	 const QString multiSafe[26]={
				"N",
				"DT",
				"CUPMIXIV",
				"A0A",
				"A0R",
				"A11",
				"A12",
				"A2",
				"A0AE",
				"A0RE",
				"RE",
				"R0",
				"C01",
				"C02",
				"X1",
				"U1",
				"X2",
				"U2",
				"X3",
				"U3",
				"X4",
				"U4",
				"X5",
				"U5",
				"X6",
				"U6"};
				
GlobalData::GlobalData()
{
  variablesNr=26;
}

void GlobalData::read()
{
  int i,j;
  QString entryName;
  KConfig *config=KGlobal::config();

	  config->setGroup("Parameters");
	  a0a = config->readDoubleNumEntry("A0A",0);
	  a0r = config->readDoubleNumEntry("A0R",0);
	  a11 = config->readDoubleNumEntry("A11",2);
	  a12 = config->readDoubleNumEntry("A12",0);
	  a2 = config->readDoubleNumEntry("A2",0);
	  d = config->readDoubleNumEntry("D",1.44e-9);
	  u = config->readDoubleNumEntry("U",0);
	  l = config->readDoubleNumEntry("L",112.2);
	  r0 = config->readDoubleNumEntry("r0",0.14);
	  k1 = config->readDoubleNumEntry("k1",0);
	  k2 = config->readDoubleNumEntry("k2",0);
	  c0 = config->readDoubleNumEntry("C0",1);
	  vd = config->readDoubleNumEntry("Vd",0);
	  a0ae = config->readDoubleNumEntry("A0Ae",a0a);
	  a0re = config->readDoubleNumEntry("A0Re",a0r);
	  re = config->readDoubleNumEntry("RE",100);
	  c01 = config->readDoubleNumEntry("C01",1);
	  c02 = config->readDoubleNumEntry("C02",0);
	  config->setGroup("Files");
	  solverPath = config->readEntry("SolverPath","~");
	  initialConditionFileName = config->readEntry("InitialCondition","");
	  outputFileName = config->readEntry("OutputFile","");
	  cupmixingAverageFileName = config->readEntry("CupmixAverage","");
	  plotFileName = config->readEntry("PlotFile","");
	  plotOptions = config->readEntry("PlotOptions","-color -noframe -nodate");
	  measurementDataFileName = config->readEntry("MeasurementFile","");
	  measurementPlotFileName = config->readEntry("MeasurementPlotFile","");
	  timeSeriesPlotFileName = config->readEntry("TimeSeriesPlotFile","");
	  	
	  config->setGroup("ExecutionParameters");
	  deltat = config->readDoubleNumEntry("deltat",0.5);
	  cupmixiv = config->readDoubleNumEntry("CupmixingIntervall",0.5);
	  deltatstar = config->readDoubleNumEntry("deltatstar",0.5);
	  n=config->readUnsignedLongNumEntry("Iterations",20);
	  averagetype=config->readLongNumEntry("AverageType",1);
	  flow = config->readLongNumEntry("FlowType",0);
	  velocity = config->readBoolEntry("UseDistribution",false);
	
	  config->setGroup("MultipleRunParameters");
	  runsNumber=config->readLongNumEntry("RunsNumber",2);
	  for(i=0;i<runsNumber;i++) {
	    for(j=0;j<variablesNr;j++) {
	      entryName=multiSafe[j];
	      entryName.append("_");
	      entryName.append(QString::number(i));
	      multipleRunVariables[i][j] = config->readDoubleNumEntry(entryName,1);
	    }
	  }
	
	  config->setGroup("Locations");
	  locationsNr=config->readLongNumEntry("Locations",0);
	  for(i=0;i<locationsNr;i++) {
	    entryName="Loc";
	    entryName.append(QString::number(i));
	    locations[i]=config->readDoubleNumEntry(entryName,0.1);
	    entryName.append("Label");
	    locationLabels[i]=config->readEntry(entryName,QString::number(i));
	  }	
	  checkEssentials();
}

void GlobalData::checkEssentials()
{
	essentialsAvailable=true;
  if (deltat==0) essentialsAvailable=false;
  if (solverPath.isEmpty()) essentialsAvailable=false;
  if (initialConditionFileName.isEmpty()) essentialsAvailable=false;
}

void GlobalData::write()
{
  int i,j;
  QString entryName;
  KConfig *config=KGlobal::config();

		 config->setGroup("Parameters");
		 config->writeEntry("A0A",a0a);
		 config->writeEntry("A0R",a0r);
		 config->writeEntry("A11",a11);
		 config->writeEntry("A12",a12);
		 config->writeEntry("A2",a2);
		 config->writeEntry("D",d);
	   config->writeEntry("U",u);
	   config->writeEntry("L",l);
	   config->writeEntry("r0",r0);
	   config->writeEntry("k1",k1);
	   config->writeEntry("k2",k2);
	   config->writeEntry("C0",c0);
	   config->writeEntry("Vd",vd);
	   config->writeEntry("A0Ae",a0ae);
	   config->writeEntry("A0Re",a0re);
	   config->writeEntry("RE",re);
	   config->writeEntry("C01",c01);
	   config->writeEntry("C02",c02);
		 config->setGroup("Files");
	   config->writeEntry("SolverPath",solverPath.prettyURL());
	   config->writeEntry("InitialCondition",initialConditionFileName.prettyURL());
	   config->writeEntry("OutputFile",outputFileName.prettyURL());
	   config->writeEntry("CupmixAverage",cupmixingAverageFileName.prettyURL());
	   config->writeEntry("PlotFile",plotFileName.prettyURL());
	   config->writeEntry("PlotOptions",plotOptions);
	   config->writeEntry("MeasurementFile",measurementDataFileName.prettyURL());
	   config->writeEntry("MeasurementPlotFile",measurementPlotFileName.prettyURL());
	   config->writeEntry("TimeSeriesPlotFile",timeSeriesPlotFileName.prettyURL());
	   config->setGroup("ExecutionParameters");
     config->writeEntry("deltat",deltat);
     config->writeEntry("deltatstar",deltatstar);
	   config->writeEntry("CupmixingIntervall",cupmixiv);
	   config->writeEntry("Iterations",n);
	   config->writeEntry("AverageType",averagetype);
	   config->writeEntry("FlowType",flow);
	   config->writeEntry("UseDistribution",velocity);
	   config->setGroup("MultipleRunParameters");
	   config->writeEntry("RunsNumber",runsNumber);
	   for(i=0;i<runsNumber;i++) {
	     for(j=0;j<variablesNr;j++) {
	       entryName=multiSafe[j];
	       entryName.append("_");
	       entryName.append(QString::number(i));
	       config->writeEntry(entryName,multipleRunVariables[i][j]);
	     }
	   }
	   config->setGroup("Locations");
	   config->writeEntry("Locations",locationsNr);
	   for(i=0;i<locationsNr;i++) {
	     entryName="Loc";
	     entryName.append(QString::number(i));
	     config->writeEntry(entryName,locations[i]);
	     entryName.append("Label");
	     config->writeEntry(entryName,locationLabels[i]);
	  }				
}

